/* All Rights reserved */

#import "Document.h"

@implementation Document

- (NSDragOperation) tableView: (NSTableView *) view
                    validateDrop: (id <NSDraggingInfo>) info
                    proposedRow: (int) row
                    proposedDropOperation: (NSTableViewDropOperation) operation
{
  if (row > [records count])
    return NSDragOperationNone;

  if (nil == [info draggingSource]) // From other application
    {
      return NSDragOperationNone;
    }
  else if (tableView == [info draggingSource]) // From self
    {
      return NSDragOperationNone;
    }
  else 
    {
      [view setDropRow: row dropOperation: NSTableViewDropAbove];
      return NSDragOperationCopy;
    }
}

- (BOOL) tableView: (NSTableView *) view
         acceptDrop: (id <NSDraggingInfo>) info
         row: (int) row
         dropOperation: (NSTableViewDropOperation) operation
{
  NSPasteboard *pboard = [info draggingPasteboard];
  NSData *data = [pboard dataForType: @"NSGeneralPboardType"];

  if (row > [records count])
    return NO;

  if (nil == [info draggingSource]) // From other application
    {
      return NO;
    }
  else if (tableView == [info draggingSource]) // From self
    {
      return NO;
    }
  else
    {
      id object = [NSUnarchiver unarchiveObjectWithData: data];
      [records insertObject: object atIndex: row];
      [tableView reloadData];

      return YES;
    }
  return NO;
}

- (BOOL) tableView: (NSTableView *) view
         writeRows: (NSArray *) rows
         toPasteboard: (NSPasteboard *) pboard
{
  id object = [records objectAtIndex: [[rows lastObject] intValue]];
  NSData *data = [NSArchiver archivedDataWithRootObject: object];

  [pboard declareTypes: [NSArray arrayWithObject: @"NSGeneralPboardType"]
                 owner: nil];
  [pboard setData: data forType: @"NSGeneralPboardType"];
  return YES;
}

- (NSData *) dataRepresentationOfType: (NSString *) type
{
  if (type == nil)
    type = @"mon";
  if ([type isEqualToString: @"mon"])
    return [[records description] dataUsingEncoding: [NSString defaultCStringEncoding]];
  else
    return nil;
}

- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) type
{
  if ([type isEqualToString: @"mon"])
    {
       NSString *string = [[NSString alloc] initWithData: data 
                                                encoding: [NSString defaultCStringEncoding]];
       [records setArray: [string propertyList]];
       RELEASE(string);
       return YES;
    }
  return NO;
}

- (id) init
{
  self = [super init];
  records = [NSMutableArray new];
  return self;
}

- (void) dealloc
{
  RELEASE(records);
  [super dealloc];
}

- (void) windowControllerDidLoadNib: (NSWindowController *) controller
{
  NSTableColumn *column;
  NSArray *columns = [tableView tableColumns];

  column = [columns objectAtIndex: 0];
  [column setWidth: 100];
  [column setEditable: YES];
  [column setResizable: YES];
  [column setIdentifier: @"date"];
  [[column headerCell] setStringValue: @"Date"];

  column = [columns objectAtIndex: 1];
  [column setWidth: 100];
  [column setEditable: YES];
  [column setResizable: YES];
  [column setIdentifier: @"item"];
  [[column headerCell] setStringValue: @"Item"];

  column = [[NSTableColumn alloc] initWithIdentifier: @"amount"];
  [column setWidth: 100];
  [column setEditable: YES];
  [column setResizable: YES];
  [[column headerCell] setStringValue: @"Amount"];
  [tableView addTableColumn: column];
  RELEASE(column);

  [tableView sizeLastColumnToFit];
  [tableView setAutoresizesAllColumnsToFit: YES];
 
  [tableView registerForDraggedTypes: [NSArray arrayWithObjects: NSGeneralPboardType, nil]];
}

- (int) numberOfRowsInTableView: (NSTableView *) view
{
  return [records count] + 1;
}

- (id) tableView: (NSTableView *) view
       objectValueForTableColumn: (NSTableColumn *) column
      row: (int) row
{
  if (row >= [records count])
    {
      return @"";
    }
  else
    {
      return [[records objectAtIndex: row] objectForKey: [column identifier]];
    }
}

- (void) tableView: (NSTableView *) view
       setObjectValue: (id) object
       forTableColumn: (NSTableColumn *) column
       row: (int) row
{
  if (row >= [records count])
    {
      [records addObject: [NSMutableDictionary new]];
    }
  [[records objectAtIndex: row] setObject: object
                                   forKey: [column identifier]];
  [tableView reloadData];
} 

- (NSString *) windowNibName
{
  return @"Document.gorm";
}

@end
